from dcentrapi.common import get_dapi_version


class Base:
    def __init__(self, stage, username=None, key=None):
        self.__version__ = get_dapi_version()

        if stage == "develop":
            self.headers = {"Authorization": username + "," + key}
            self.url = "https://test-api.dcentralab.com/"
            self.web3index_url = "https://test-api.web3index.info/"
        if stage == "staging":
            self.headers = {"Authorization": username + "," + key}
            self.url = "https://staging.dcentralab.com/"
            self.web3index_url = "https://staging-api.web3index.info/"
        if stage == "main":
            self.headers = {"Authorization": username + "," + key}
            self.url = "https://main-api.dcentralab.com/"
            self.web3index_url = "https://api.web3index.info/"



from dcentrapi.Base import Base
from dcentrapi.common import DapiError
from dcentrapi.modules.requests_dappi import requests_get


class HackMitigation(Base):

    def are_addresses_blacklisted(self, addresses: [str]):
        url = self.url + "generic_freeze_signal/are_addresses_blacklisted"
        data = {
            "addresses": addresses,
        }
        response = requests_get(url, json=data, headers=self.headers)
        try:
            return response.json()
        except Exception as e:
            return DapiError(response=response, exception=e)



from dcentrapi.Base import Base
from dcentrapi.modules.requests_dappi import requests_get


class Web3Index(Base):
    def get_pairs(self, network_name: str, token_address: str):
        url = self.web3index_url + "pairs" + f"/{network_name}/{token_address}"
        response = requests_get(url, headers=self.headers)
        return response.json()

    def get_factories(self):
        url = self.web3index_url + "factories"
        response = requests_get(url, headers=self.headers)
        return response.json()

    def get_token_price_snapshot(self, info):
        # Currently info is token symbol (str), e.g. "XCAD"
        # In future, might also have the base token id (int)
        url = self.web3index_url + "token_price_snapshot" + f"/{info}"
        response = requests_get(url, headers=self.headers)
        return response.json()



from dcentrapi.Base import Base
from dcentrapi.eventPolling import EventPolling
from dcentrapi.rpcAggregation import RpcAggregation
from dcentrapi.gasMonitor import GasMonitor
from dcentrapi.web3Index import Web3Index
from dcentrapi.hackMitigation import HackMitigation
from dcentrapi.modules.requests_dappi import requests_get, requests_post
# from dcentrapi.merkleTree import MerkleTree, MerkleTreeNode



from dcentrapi import MerkleTree
from dcentrapi.merkleTree import verify_leaf

tree = MerkleTree("develop", "Chainport", 'f3taga-mHNLlv8KDh_HE76oOGyU')
leaf1 = ("0x9D9fF3A3A147fb5F96d773E244BAc63Ffa1F21C9", 307835000000000028, 1)
leaf2 = ("0x9D9fF3A3A147fb5F96d773E244BAc63Ffa1F21C9", 99366666666666676687, 2)
leaf3 = ("0x9D9fF3A3A147fb5F96d773E244BAc63Ffa1F21C9", 38500000000000003, 3)
leaves = [leaf1, leaf2, leaf3]
encoding = "(address,uint256,uint256)"
root, layers = tree.build_tree(leaves, encoding)
print(root.hashValue.hex())


leaf_not_in_tree = ("0x9D9fF3A3A147fb5F96d773E244BAc63Ffa1F21C9", 38500000000000003, 4)
proof1 = tree.get_proof(leaf=leaf1)
proof1 = [bytes(p.hashValue) for p in proof1]
assert verify_leaf(root=root.hashValue.hex(), leaf=leaf1, proof=proof1, encoding=encoding)
assert verify_leaf(root=root.hashValue.hex(), leaf=leaf_not_in_tree, proof=proof1, encoding=encoding) is False

